@@ -6,7 +6,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: si.c 1.25 2008/03/05 17:00:55 kls Exp $
+ *   $Id: si.c 2.3 2010/11/01 15:24:32 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -311,6 +311,11 @@ static const char *CharacterTables2[] = {
 static const char *SystemCharacterTable = NULL;
 bool SystemCharacterTableIsSingleByte = true;
 
+bool systemCharacterTableIsSingleByte(void)
+{
+  return SystemCharacterTableIsSingleByte;
+}
+
 bool SetSystemCharacterTable(const char *CharacterTable) {
    if (CharacterTable) {
       for (unsigned int i = 0; i < NumEntries(CharacterTables1); i++) {
@@ -335,11 +340,7 @@ bool SetSystemCharacterTable(const char *CharacterTable) {
    return false;
 }
 
-// Determines the character table used in the given buffer and returns
-// a string indicating that table. If no table can be determined, the
-// default ISO6937 is returned. If a table can be determined, the buffer
-// and length are adjusted accordingly.
-static const char *getCharacterTable(const unsigned char *&buffer, int &length, bool *isSingleByte = NULL) {
+const char *getCharacterTable(const unsigned char *&buffer, int &length, bool *isSingleByte) {
    const char *cs = "ISO6937";
    // Workaround for broadcaster stupidity: according to
    // "ETSI EN 300 468" the default character set is ISO6937. But unfortunately some
@@ -375,7 +376,7 @@ static const char *getCharacterTable(const unsigned char *&buffer, int &length,
    return cs;
 }
 
-static bool convertCharacterTable(const char *from, size_t fromLength, char *to, size_t toLength, const char *fromCode)
+bool convertCharacterTable(const char *from, size_t fromLength, char *to, size_t toLength, const char *fromCode)
 {
   if (SystemCharacterTable) {
      iconv_t cd = iconv_open(SystemCharacterTable, fromCode);
@@ -605,6 +606,9 @@ Descriptor *Descriptor::getDescriptor(CharArray da, DescriptorTagDomain domain,
          case ExtensionDescriptorTag:
             d=new ExtensionDescriptor();
             break;
+         case RegistrationDescriptorTag:
+            d=new RegistrationDescriptor();
+            break;
 
          //note that it is no problem to implement one
          //of the unimplemented descriptors.
@@ -613,7 +617,6 @@ Descriptor *Descriptor::getDescriptor(CharArray da, DescriptorTagDomain domain,
          case VideoStreamDescriptorTag:
          case AudioStreamDescriptorTag:
          case HierarchyDescriptorTag:
-         case RegistrationDescriptorTag:
          case DataStreamAlignmentDescriptorTag:
          case TargetBackgroundGridDescriptorTag:
          case VideoWindowDescriptorTag:
@@ -720,6 +723,7 @@ Descriptor *Descriptor::getDescriptor(CharArray da, DescriptorTagDomain domain,
             break;
       }
       break;
+   default: ; // unknown domain, nothing to do
    }
    d->setData(da);
    return d;
